#include <windows.h>
#include <windowsx.h>
#include <streams.h>
#include <commctrl.h>
#include <olectl.h>
#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <tchar.h>
#include "resource.h"
#include "ave_uids.h"
#include "inf_ave.h"
#include "avienc.h"
#include "ave_prop.h"


//
// CreateInstance
//
// Used by the ActiveMovie base classes to create instances
//
CUnknown *CAVIEncoderProperties::CreateInstance(LPUNKNOWN lpunk, HRESULT *phr)
{
    CUnknown *punk = new CAVIEncoderProperties(lpunk, phr);
    if (punk == NULL) {
	*phr = E_OUTOFMEMORY;
    }
    return punk;

} // CreateInstance


//
// Constructor
//
CAVIEncoderProperties::CAVIEncoderProperties(LPUNKNOWN pUnk, HRESULT *phr) :
    CBasePropertyPage(NAME("AVIEncoder Property Page"),
                      pUnk,IDD_AVE_PROP,IDS_TITLE),
    m_pIPAviEnc(NULL),
    m_bIsInitialized(FALSE)
{
    ASSERT(phr);

} // (Constructor)


//
// OnReceiveMessage
//
// Handles the messages for our property window
//
BOOL CAVIEncoderProperties::OnReceiveMessage(HWND hwnd,
                                          UINT uMsg,
                                          WPARAM wParam,
                                          LPARAM lParam)
{
    switch (uMsg)
    {
        case WM_COMMAND:
        {
           if(LOWORD(wParam)==IDC_DEFAULT) 
		   {
				setFRate(0.0);
		   }
           else if(LOWORD(wParam)==IDC_PAL) 
		   {
				setFRate(25.0);
		   }
           else if(LOWORD(wParam)==IDC_NTSC) 
		   {
				setFRate(23.976);
		   }
            if (m_bIsInitialized)
            {
                m_bDirty = TRUE;
                if (m_pPageSite)
                {
                    m_pPageSite->OnStatusChange(PROPPAGESTATUS_DIRTY);
                }
            }
            return (LRESULT) 1;
        }

    }
    return CBasePropertyPage::OnReceiveMessage(hwnd,uMsg,wParam,lParam);

} // OnReceiveMessage


//
// OnConnect
//
// Called when we connect to a transform filter
//
HRESULT CAVIEncoderProperties::OnConnect(IUnknown *pUnknown)
{
    ASSERT(m_pIPAviEnc == NULL);

    HRESULT hr = pUnknown->QueryInterface(IID_IIPAVIEnc, (void **) &m_pIPAviEnc);
    if (FAILED(hr)) {
        return E_NOINTERFACE;
    }

    ASSERT(m_pIPAviEnc);

    // Get the initial image FX property
    m_pIPAviEnc->get_IPAVIEnc(&m_top, &m_bottom, &m_left, &m_right, &m_width, &m_height, &m_deinterlace, &m_frate, m_filename, &m_aviopts);

    m_bIsInitialized = FALSE ;
    return NOERROR;

} // OnConnect


//
// OnDisconnect
//
// Likewise called when we disconnect from a filter
//
HRESULT CAVIEncoderProperties::OnDisconnect()
{
    // Release of Interface after setting the appropriate old effect value

    if (m_pIPAviEnc == NULL) {
        return E_UNEXPECTED;
    }

    m_pIPAviEnc->Release();
    m_pIPAviEnc = NULL;
    return NOERROR;

} // OnDisconnect


//
// OnActivate
//
// We are being activated
//
HRESULT CAVIEncoderProperties::OnActivate()
{
	SetDlgItemInt(m_Dlg, IDC_TOP, m_top, FALSE);
	SetDlgItemInt(m_Dlg, IDC_BOTTOM, m_bottom, FALSE);
	SetDlgItemInt(m_Dlg, IDC_LEFT, m_left, FALSE);
	SetDlgItemInt(m_Dlg, IDC_RIGHT, m_right, FALSE);
	SetDlgItemInt(m_Dlg, IDC_WIDTH, m_width, FALSE);
	SetDlgItemInt(m_Dlg, IDC_HEIGHT, m_height, FALSE);
	setFRate(m_frate);
	SetDlgItemText(m_Dlg, IDC_FILENAME, m_filename);
	CheckDlgButton(m_Dlg, IDC_DEINTERLACE, (m_deinterlace==0) ? BST_UNCHECKED : BST_CHECKED);
	CheckDlgButton(m_Dlg, IDC_AVIOPTS, (m_aviopts==0) ? BST_UNCHECKED : BST_CHECKED);
    m_bIsInitialized = TRUE;
    return NOERROR;
} // OnActivate

void CAVIEncoderProperties::setFRate(double frate)
{
	TCHAR	tbuf[200];

	m_frate=frate;
	_stprintf(tbuf, TEXT("%.4f"), m_frate);
	SetDlgItemText(m_Dlg, IDC_FRAMERATE, tbuf);
}

//
// OnDeactivate
//
// We are being deactivated
//
HRESULT CAVIEncoderProperties::OnDeactivate(void)
{
    ASSERT(m_pIPAviEnc);
    m_bIsInitialized = FALSE;
    GetControlValues();
    return NOERROR;

} // OnDeactivate


//
// OnApplyChanges
//
// Apply any changes so far made
//
HRESULT CAVIEncoderProperties::OnApplyChanges()
{
    GetControlValues();
    m_pIPAviEnc->put_IPAVIEnc(m_top, m_bottom, m_left, m_right, m_width, m_height, m_deinterlace, m_frate, m_filename, m_aviopts);

    return NOERROR;
} // OnApplyChanges


void CAVIEncoderProperties::GetControlValues()
{
	TCHAR	tbuf[200];

    ASSERT(m_pIPAviEnc);
	BOOL success;

	m_top=GetDlgItemInt(m_Dlg, IDC_TOP, &success, FALSE);
	m_bottom=GetDlgItemInt(m_Dlg, IDC_BOTTOM, &success, FALSE);
	m_left=GetDlgItemInt(m_Dlg, IDC_LEFT, &success, FALSE);
	m_right=GetDlgItemInt(m_Dlg, IDC_RIGHT, &success, FALSE);
	m_width=GetDlgItemInt(m_Dlg, IDC_WIDTH, &success, FALSE);
	m_height=GetDlgItemInt(m_Dlg, IDC_HEIGHT, &success, FALSE);
	GetDlgItemText(m_Dlg, IDC_FRAMERATE, tbuf, 200);
	m_frate=atof(tbuf);
	GetDlgItemText(m_Dlg, IDC_FILENAME, m_filename, MAX_PATH);
	m_deinterlace=(IsDlgButtonChecked(m_Dlg, IDC_DEINTERLACE)==BST_CHECKED) ? 1 : 0;
	m_aviopts=(IsDlgButtonChecked(m_Dlg, IDC_AVIOPTS)==BST_CHECKED) ? 1 : 0;
}
