class CAVIEncoderInputPin;
class CAVIEncoder;
class CAVIEncoderFilter;

class CAVIEncoderFilter : public CBaseFilter
{
    CAVIEncoder * const m_pAVIEncoder;

public:

    // Constructor
    CAVIEncoderFilter(CAVIEncoder *pAVIEncoder,
                LPUNKNOWN pUnk,
                CCritSec *pLock,
                HRESULT *phr);

    // Pin enumeration
    CBasePin * GetPin(int n);
    int GetPinCount();

    // Open and close the file as necessary
    STDMETHODIMP Run(REFERENCE_TIME tStart);
    STDMETHODIMP Pause();
    STDMETHODIMP Stop();
};


//  Pin object

class CAVIEncoderInputPin : public CRenderedInputPin
{
    CAVIEncoder    * const m_pAVIEncoder;           // Main renderer object
    CCritSec * const m_pReceiveLock;    // Sample critical section
    REFERENCE_TIME m_tLast;             // Last sample receive time

public:

    CAVIEncoderInputPin(CAVIEncoder *pAVIEncoder,
                  LPUNKNOWN pUnk,
                  CBaseFilter *pFilter,
                  CCritSec *pLock,
                  CCritSec *pReceiveLock,
                  HRESULT *phr);

    // Do something with this media sample
    STDMETHODIMP Receive(IMediaSample *pSample);
    STDMETHODIMP EndOfStream(void);
    STDMETHODIMP ReceiveCanBlock();
    HRESULT WriteStringInfo(IMediaSample *pSample);

    // Check if the pin can support this specific proposed type and format
    HRESULT CheckMediaType(const CMediaType *);

    // Break connection
    HRESULT BreakConnect();

    // Track NewSegment
    STDMETHODIMP NewSegment(REFERENCE_TIME tStart,
                            REFERENCE_TIME tStop,
                            double dRate);
};


//  CAVIEncoder object which has filter and pin members

class CAVIEncoder : public CUnknown, public IIPAVIEnc, public ISpecifyPropertyPages
{
    friend class CAVIEncoderFilter;
    friend class CAVIEncoderInputPin;

    CAVIEncoderFilter *m_pFilter;         // Methods for filter interfaces
    CAVIEncoderInputPin *m_pPin;          // A simple rendered input pin
    CCritSec m_Lock;                // Main renderer critical section
    CCritSec m_ReceiveLock;         // Sublock for received samples
    CPosPassThru *m_pPosition;      // Renderer position controls

	int m_top;						// the top crop
	int m_bottom;					// the bottom crop
	int m_left;						// the left crop
	int m_right;					// the right crop
	int m_width;					// the width to scale to
	int m_height;					// the height to scale to
	int m_deinterlace;				// if !=0 deinterlace video 
	double m_frate;					// the output framerate
	TCHAR m_filename[MAX_PATH];		// the output file name
	int m_aviopts;					// if !=0 avi options are displayed
	int m_aviinit;					// is avi init done
	long m_startFrame;				// the frame to start encoding at (inclusive)
	long m_stopFrame;				// the frame to stop encoding at (exclusive) -1 for encoding until end 

public:

    DECLARE_IUNKNOWN

    CAVIEncoder(LPUNKNOWN pUnk, HRESULT *phr);
    ~CAVIEncoder();

    static CUnknown * WINAPI CreateInstance(LPUNKNOWN punk, HRESULT *phr);

    // These implement the custom IIPAVIEnc interface
	STDMETHODIMP get_IPAVIEnc(int *top, int *bottom, int *left, int *right, int *width, int *height, int *deint, double *frate, LPTSTR filename, int *showopt);
	STDMETHODIMP put_IPAVIEnc(int top, int bottom, int left, int right, int width, int height, int deint, double frate, LPTSTR filename, int showopt);
	STDMETHODIMP get_FrameCount(long *frames);
	STDMETHODIMP close_AVI();

    // ISpecifyPropertyPages interface
    STDMETHODIMP GetPages(CAUUID *pPages);

private:

    // Overriden to say what interfaces we support where
    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);
};

