#include <windows.h>
#include <windowsx.h>
#include <streams.h>
#include <commctrl.h>
#include <olectl.h>
#include <memory.h>
#include <mmsystem.h>
#include <stdlib.h>
#include <stdio.h>
#include <tchar.h>
#include <vfw.h>

extern HRESULT initAVI(LPSTR filename, LPBITMAPINFOHEADER lpbi, DWORD dwRate, DWORD dwScale);
extern HRESULT writeAVI(BYTE *data, DWORD len);
extern void closeAVI();

static void putAVIOptsIntoRegistry(AVICOMPRESSOPTIONS *opts);
static BOOL getAVIOptsFromRegistry(AVICOMPRESSOPTIONS *opts);

static AVISTREAMINFO strhdr;
static PAVIFILE pfile = NULL;
static PAVISTREAM ps = NULL, psCompressed = NULL;
static AVICOMPRESSOPTIONS opts;
static AVICOMPRESSOPTIONS FAR * aopts[1] = {&opts};
LONG	frameNr;


HRESULT initAVI(LPSTR filename, LPBITMAPINFOHEADER lpbi, DWORD dwRate, DWORD dwScale, BOOL showOpts)
{
	HRESULT hr;

	frameNr=0;
	AVIFileInit();
	hr = AVIFileOpen(&pfile,		    // returned file pointer
		       filename,		            // file name
		       OF_WRITE | OF_CREATE,	    // mode to open file with
		       NULL);			    // use handler determined
						    // from file extension....
	if (hr != AVIERR_OK)
	{
		closeAVI();
		return hr;
	}

	// Fill in the header for the video stream....

	// The video stream will run in 15ths of a second....

	_fmemset(&strhdr, 0, sizeof(strhdr));
	strhdr.fccType                = streamtypeVIDEO;// stream type
	strhdr.fccHandler             = 0;
	strhdr.dwScale                = dwScale;
	strhdr.dwRate                 = dwRate;		    // 15 fps
	strhdr.dwSuggestedBufferSize  = lpbi->biSizeImage;
	SetRect(&strhdr.rcFrame, 0, 0, lpbi->biWidth, lpbi->biHeight);

	// And create the stream;
	hr = AVIFileCreateStream(pfile,		    // file pointer
			         &ps,		    // returned stream pointer
			         &strhdr);	    // stream header
	if (hr != AVIERR_OK) {
		closeAVI();
		return hr;
	}

	_fmemset(&opts, 0, sizeof(opts));
	if(getAVIOptsFromRegistry(&opts)==FALSE)
		showOpts=TRUE;

	if(showOpts)
	{
		if(!AVISaveOptions(NULL, 0, 1, &ps, (LPAVICOMPRESSOPTIONS FAR *) &aopts))
		{
			closeAVI();
			return 1;
		}
	}

	putAVIOptsIntoRegistry(&opts);

	hr = AVIMakeCompressedStream(&psCompressed, ps, &opts, NULL);
	if (hr != AVIERR_OK) {
		closeAVI();
		return hr;
	}

	hr = AVIStreamSetFormat(psCompressed, 0, lpbi, lpbi->biSize + lpbi->biClrUsed * sizeof(RGBQUAD));
	if (hr != AVIERR_OK) {
		closeAVI();
		return hr;
	}

	return NOERROR;
}

#define REG_KEY		TEXT("Software\\AVIEnc")
#define	REG_PARAM	TEXT("codec_opts")

static void putAVIOptsIntoRegistry(AVICOMPRESSOPTIONS *opts)
{
	char		*buf=NULL;
	HKEY		hKey=NULL;
	DWORD		err, size;

	err=RegCreateKeyEx(HKEY_LOCAL_MACHINE, REG_KEY, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL); 
	if(err!=ERROR_SUCCESS)
		return;

	size=sizeof(AVICOMPRESSOPTIONS)+opts->cbFormat+opts->cbParms;
	buf=(char *)CoTaskMemAlloc(size);
	if(buf!=NULL)
	{
		char	*b;

		b=buf;
		CopyMemory(b, opts, sizeof(AVICOMPRESSOPTIONS));
		b+=sizeof(AVICOMPRESSOPTIONS);
		CopyMemory(b, opts->lpFormat, opts->cbFormat);
		b+=opts->cbFormat;
		CopyMemory(b, opts->lpParms, opts->cbParms);

		RegSetValueEx(hKey, REG_PARAM, 0, REG_BINARY, (CONST BYTE *)buf, size);
	}

	if(buf)
		CoTaskMemFree(buf);

	RegCloseKey(hKey);
}

static BOOL getAVIOptsFromRegistry(AVICOMPRESSOPTIONS *opts)
{
	char		*buf=NULL;
	HKEY		hKey=NULL;
	DWORD		err, size;
	BOOL		ret=FALSE;

	err=RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_KEY, 0, KEY_READ, &hKey); 
	if(err!=ERROR_SUCCESS)
		return FALSE;

	size=0;
	err=RegQueryValueEx(hKey, REG_PARAM, 0, NULL, NULL, &size);
	if(err!=ERROR_SUCCESS)
		return FALSE;

	buf=(char *)CoTaskMemAlloc(size);
	if(buf!=NULL)
	{
		err=RegQueryValueEx(hKey, REG_PARAM, 0, NULL, (BYTE *)buf, &size);
		if(err==ERROR_SUCCESS)
		{
			char	*b;

			b=buf;
			CopyMemory(opts, b, sizeof(AVICOMPRESSOPTIONS));
			b+=sizeof(AVICOMPRESSOPTIONS);

			opts->lpFormat=CoTaskMemAlloc(opts->cbFormat);
			CopyMemory(opts->lpFormat, b, opts->cbFormat);
			b+=opts->cbFormat;

			opts->lpParms=CoTaskMemAlloc(opts->cbParms);
			CopyMemory(opts->lpParms, b, opts->cbParms);

			ret=TRUE;
		}
	}

	if(buf)
		CoTaskMemFree(buf);

	RegCloseKey(hKey);

	return ret;
}

HRESULT writeAVI(BYTE *data, DWORD len)
{
	HRESULT	hr;


	if(psCompressed)
	{
		hr = AVIStreamWrite(psCompressed,	// stream pointer
				frameNr++,		// time of this frame
				1,				// number to write
				data,
				len,	// size of this frame
				AVIIF_KEYFRAME,			 // flags....
				NULL,
				NULL);	
	}

	return hr;
}

void closeAVI()
{
	if (ps)
		AVIStreamClose(ps);
	ps=NULL;

	if (psCompressed)
		AVIStreamClose(psCompressed);
	psCompressed=NULL;

	if (pfile)
		AVIFileClose(pfile);
	pfile=NULL;

	AVIFileExit();
}

