//==========================================================================;
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1992 - 1996  Microsoft Corporation.  All Rights Reserved.
//
//--------------------------------------------------------------------------;

class CBinDumpInputPin;
class CBinDump;
class CBinDumpFilter;


// Main filter object

class CBinDumpFilter : public CBaseFilter
{
    CBinDump * const m_pDump;

public:

    // Constructor
    CBinDumpFilter(CBinDump *pDump,
                LPUNKNOWN pUnk,
                CCritSec *pLock,
                HRESULT *phr);

    // Pin enumeration
    CBasePin * GetPin(int n);
    int GetPinCount();

    // Open and close the file as necessary
    STDMETHODIMP Run(REFERENCE_TIME tStart);
    STDMETHODIMP Pause();
    STDMETHODIMP Stop();
};


//  Pin object

class CBinDumpInputPin : public CRenderedInputPin
{
    CBinDump    * const m_pDump;           // Main renderer object
    CCritSec * const m_pReceiveLock;    // Sample critical section
    REFERENCE_TIME m_tLast;             // Last sample receive time

public:

    CBinDumpInputPin(CBinDump *pDump,
                  LPUNKNOWN pUnk,
                  CBaseFilter *pFilter,
                  CCritSec *pLock,
                  CCritSec *pReceiveLock,
                  HRESULT *phr);

    // Do something with this media sample
    STDMETHODIMP Receive(IMediaSample *pSample);
    STDMETHODIMP EndOfStream(void);
 //   void WriteString(TCHAR *pString);
	void WriteBinary(PBYTE pbyte, LONG length);

    // Check if the pin can support this specific proposed type and format
    HRESULT CheckMediaType(const CMediaType *);

    // Break connection
    HRESULT BreakConnect();

    // Track NewSegment
    STDMETHODIMP NewSegment(REFERENCE_TIME tStart,
                            REFERENCE_TIME tStop,
                            double dRate);
};


//  CBinDump object which has filter and pin members

class CBinDump : public CUnknown, public IFileSinkFilter
{
    friend class CBinDumpFilter;
    friend class CBinDumpInputPin;

    CBinDumpFilter *m_pFilter;         // Methods for filter interfaces
    CBinDumpInputPin *m_pPin;          // A simple rendered input pin
    CCritSec m_Lock;                // Main renderer critical section
    CCritSec m_ReceiveLock;         // Sublock for received samples
    CPosPassThru *m_pPosition;      // Renderer position controls
    HANDLE m_hFile;                 // Handle to file for dumping
    LPOLESTR m_pFileName;           // The filename where we dump to

public:

    DECLARE_IUNKNOWN

    CBinDump(LPUNKNOWN pUnk, HRESULT *phr);
    ~CBinDump();

    static CUnknown *CreateInstance(LPUNKNOWN punk, HRESULT *phr);

    // Setup helper
    LPAMOVIESETUP_FILTER GetSetupData();

    // Implements the IFileSourceFilter interface

    STDMETHODIMP SetFileName(LPCOLESTR pszFileName,const AM_MEDIA_TYPE *pmt);
    STDMETHODIMP GetCurFile(LPOLESTR * ppszFileName,AM_MEDIA_TYPE *pmt);
	void WriteMediaType(const CMediaType *pmt);

private:

    // Overriden to say what interfaces we support where
    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);

    // Open and write to the file
    HRESULT OpenFile();
    HRESULT CloseFile();
	void WriteBinary(PBYTE pbyte, LONG length);
};

