//==========================================================================;
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1996 - 1998  Microsoft Corporation.  All Rights Reserved.
//
//--------------------------------------------------------------------------;

#include <streams.h>
#include <initguid.h>   // Make our GUID get defined
#include "multfilesrc.h"
#include "mfs_prop_file.h"
#include "mfs_prop_media.h"

AMOVIESETUP_MEDIATYPE sudPinTypes =
{
    &MEDIATYPE_NULL,            // Major type
    &MEDIASUBTYPE_NULL          // Minor type
};

AMOVIESETUP_PIN sudPins =
{
    L"Output",                  // Pin string name
    FALSE,                      // Is it rendered
    TRUE,                       // Is it an output
    FALSE,                      // Allowed none
    FALSE,                      // Likewise many
    &CLSID_NULL,                // Connects to filter
    NULL,		                // Connects to pin
    1,                          // Number of types
    &sudPinTypes                // Pin information
};

AMOVIESETUP_FILTER sudFilter =
{
    &CLSID_MultFileSrc,					// Filter CLSID
    L"Multiple file source filter",     // String name
    MERIT_DO_NOT_USE,					// Filter merit
    1,									// Number pins
    &sudPins							// Pin details
};


//
//  Object creation stuff
//
CFactoryTemplate g_Templates[]= {
	{ L"Multiple file source filter"
	, &CLSID_MultFileSrc
	, CMultFileSrcFilter::CreateInstance
	, NULL
	, &sudFilter }
  ,
    { L"MFSrc filter Property Page - Filelist"
    , &CLSID_MFSPropertyPageFile
    , CMFSPropertiesFile::CreateInstance }
  ,
    { L"MFSrc filter Property Page - Mediatype"
    , &CLSID_MFSPropertyPageMedia
    , CMFSPropertiesMedia::CreateInstance }
};
int g_cTemplates = sizeof(g_Templates) / sizeof(g_Templates[0]);

//
// DllRegisterSever
//
// Handle the registration of this filter
//
STDAPI  DllRegisterServer()
{
    return AMovieDllRegisterServer2(TRUE);

} // DllRegisterServer


//
// DllUnregisterServer
//
STDAPI  DllUnregisterServer()
{
    return AMovieDllRegisterServer2(FALSE);

} // DllUnregisterServer


/* Create a new instance of this class */

CUnknown *CMultFileSrcFilter::CreateInstance(LPUNKNOWN pUnk, HRESULT *phr)
{
    /*  DLLEntry does the right thing with the return code and
        returned value on failure
    */
    return new CMultFileSrcFilter(pUnk, phr);
}

//
// NonDelegatingQueryInterface
//
// Override this to say what interfaces we support where
//
STDMETHODIMP CMultFileSrcFilter::NonDelegatingQueryInterface(REFIID riid, void ** ppv)
{
    CheckPointer(ppv,E_POINTER);

    // Do we have this interface

    if (riid == IID_IIPMFSrc) {
        return GetInterface((IIPMFSrc *) this, ppv);
    } else if (riid == IID_ISpecifyPropertyPages) {
        return GetInterface((ISpecifyPropertyPages *) this, ppv);
    } else if (riid == IID_IFileSourceFilter) {
        return GetInterface((IFileSourceFilter *)this, ppv);
    } else {
        return CAsyncReader::NonDelegatingQueryInterface(riid, ppv);
    }
} // NonDelegatingQueryInterface


//
// GetPages
//
// Returns the clsid's of the property pages we support
//
STDMETHODIMP CMultFileSrcFilter::GetPages(CAUUID *pPages)
{
    pPages->cElems = 2;
    pPages->pElems = (GUID *) CoTaskMemAlloc(2*sizeof(GUID));
    if (pPages->pElems == NULL) {
        return E_OUTOFMEMORY;
    }
    pPages->pElems[0] = CLSID_MFSPropertyPageFile;
    pPages->pElems[1] = CLSID_MFSPropertyPageMedia;
    return NOERROR;

} // GetPages


//
//
STDMETHODIMP CMultFileSrcFilter::get_FileList(MFS_FileList **first, LONGLONG *skip)
{
    CheckPointer(first,E_POINTER);
    CheckPointer(skip,E_POINTER);

	*first=m_first;
	*skip=m_llskip;

    return NOERROR;
} 

STDMETHODIMP CMultFileSrcFilter::put_FileList(MFS_FileList *first, LONGLONG skip)
{
    CheckPointer(first,E_POINTER);

	FreeFileList();

	m_llskip=skip;
	int				len;
	MFS_FileList	*fl, *pfl;
	
	m_first=NULL;
	while(first)
	{
        len=lstrlenW(first->Filename);

		fl=(MFS_FileList *)CoTaskMemAlloc(sizeof(MFS_FileList)+len*sizeof(WCHAR));
        if(fl!=NULL) 
		{
			fl->Next=NULL;
		    CopyMemory(fl->Filename, first->Filename, (len+1)*sizeof(WCHAR));

			if(m_first==NULL)
				m_first=fl;
			else
				pfl->Next=fl;

			pfl=fl;
        }

		first=first->Next;
	}


	HFileList	*hfl;
	hfl=createFileHandleList();
	if(hfl==NULL)
		return E_FAIL;

    m_Stream.Init(hfl, skip);

    return NOERROR;
} 

HFileList *CMultFileSrcFilter::createFileHandleList()
{
	HANDLE			hFile;
	MFS_FileList	*fl;
	HFileList		*hfl, *phfl, *hfirst=NULL;
	ULARGE_INTEGER	uliSize;

	fl=m_first;
	while(fl)
	{
		hFile=CreateFileW(fl->Filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
		if(hFile==INVALID_HANDLE_VALUE)
		{
			hfl=hfirst;
			while(hfl)
			{
				phfl=hfl->Next;

				CloseHandle(hfl->hFile);
				delete hfl;

				hfl=phfl;
			}

			return NULL;
		}

		hfl=new HFileList;
		hfl->Next=NULL;
		hfl->hFile=hFile;
		hfl->offset=0;
    
		uliSize.LowPart=GetFileSize(hFile, &uliSize.HighPart);
		hfl->length=(LONGLONG)uliSize.QuadPart;

		if(hfirst==NULL)
			hfirst=hfl;
		else
			phfl->Next=hfl;

		phfl=hfl;
		fl=fl->Next;
	}

	return hfirst;
}

STDMETHODIMP CMultFileSrcFilter::get_MediaType(CMediaType **mediaType)
{
    CheckPointer(mediaType,E_POINTER);

	*mediaType=new CMediaType(m_mt);

    return NOERROR;
} 

STDMETHODIMP CMultFileSrcFilter::put_MediaType(const CMediaType *mType)
{
    CheckPointer(mType,E_POINTER);

	m_mt=*(new CMediaType(*mType));

    return NOERROR;
} 

STDMETHODIMP CMultFileSrcFilter::put_MediaType(const AM_MEDIA_TYPE *pmtype)
{
    CheckPointer(pmtype,E_POINTER);

	m_mt=*(new CMediaType(*pmtype));

    return NOERROR;
} 


STDMETHODIMP CMultFileSrcFilter::put_MediaType(const GUID *majortype, const GUID *minortype)
{
    CheckPointer(majortype,E_POINTER);
    CheckPointer(minortype,E_POINTER);

	m_mt.SetType(majortype);
	m_mt.SetSubtype(minortype);

    return NOERROR;
} 

STDMETHODIMP CMultFileSrcFilter::load_MediaType(CMediaType **mediaType, WCHAR *filename)
{
	char		id[4];
	HANDLE		hFile;
	GUID		type, subtype, formtype;
	BYTE		*format=NULL;
	DWORD		dwW;
	ULONG		formlen, sampSize;
	BOOL		tempComp;

    CheckPointer(mediaType,E_POINTER);

	*mediaType=NULL;
    hFile=CreateFileW(filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if(hFile==INVALID_HANDLE_VALUE) 
	{
fail:
		if(format!=NULL)
			CoTaskMemFree(format);

		return E_FAIL;
	}


	ReadFile(hFile, (PVOID)id, 4, &dwW, NULL);
	if(dwW!=4 || strcmp(id, "MT0")!=0)
		goto fail;

	ReadFile(hFile, (PVOID)&type, sizeof(GUID), &dwW, NULL);
	if(dwW!=sizeof(GUID))
		goto fail;

	ReadFile(hFile, (PVOID)&subtype, sizeof(GUID), &dwW, NULL);
	if(dwW!=sizeof(GUID))
		goto fail;

	ReadFile(hFile, (PVOID)&formlen, sizeof(ULONG), &dwW, NULL);
	if(dwW!=sizeof(ULONG))
		goto fail;

	if(formlen!=0)
	{
		format=(BYTE *)CoTaskMemAlloc(formlen);
		if(format==NULL)
			goto fail;

		ReadFile(hFile, (PVOID)format, formlen, &dwW, NULL);
		if(dwW!=formlen)
			goto fail;
	}

	ReadFile(hFile, (PVOID)&formtype, sizeof(GUID), &dwW, NULL);
	if(dwW!=sizeof(GUID))
		goto fail;

	ReadFile(hFile, (PVOID)&sampSize, sizeof(ULONG), &dwW, NULL);
	if(dwW!=sizeof(ULONG))
		goto fail;

	ReadFile(hFile, (PVOID)&tempComp, sizeof(BOOL), &dwW, NULL);
	if(dwW!=sizeof(BOOL))
		goto fail;
	
	CloseHandle(hFile);

	(*mediaType)=new CMediaType(&type);
	(*mediaType)->SetSubtype(&subtype);
	(*mediaType)->SetFormat(format, formlen);
	CoTaskMemFree(format);
	(*mediaType)->SetFormatType(&formtype);
	(*mediaType)->SetSampleSize(sampSize);
	(*mediaType)->SetTemporalCompression(tempComp);

	return NOERROR;
}
